/* $LAAS: viam_acq_driver.h 2011/01/21 13:54:45 mallet $ */

/* 
 * Copyright (c) 2008,2011 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Wed Nov 12 2008
 */

#ifndef H_VIAM_ACQ_PLUGIN
#define H_VIAM_ACQ_PLUGIN

#include "viam/viamlib.h"

/** Entry point of an acquisition device driver */
#define viam_acqdev_entry	viam_acqdev_entry
#define viam_acqdev_sym		"viam_acqdev_entry"


/** Hardware filter arguments */
typedef union viam_filterargs_obj {
  struct /* anonymous */ {
    double left, top, right, bottom;
    double towidth, toheight;
  };					/**< crop, scale and the like */
  struct {
    double blue, red;			/**< white balance */
  };
  double value;				/**< single valued filters */
  viam_filterformat_t format;		/**< for format filters */
} *viam_filterargs_obj;


/** Acquisition device */
typedef struct viam_acqdev_obj {
  const char *name;

  int (*init)(viam_handle_t vh);
  int (*fini)(viam_handle_t vh);

  int (*list)(viam_handle_t vh);

  int (*attach)(viam_handle_t vh, viam_camera_t camera);
  void (*detach)(viam_handle_t vh, viam_camera_t camera);

  int (*configure)(viam_handle_t vh, viam_camera_t camera);
  int (*unconfigure)(viam_handle_t vh, viam_camera_t camera);

  int (*datatransmit)(viam_handle_t vh, viam_camera_t camera, viam_onoff_t on);
  int (*dataflush)(viam_handle_t vh, viam_camera_t camera);

  int (*capture)(viam_handle_t vh, viam_camera_t camera);
  char *(*capturedata)(viam_handle_t vh, viam_camera_t camera, size_t *bytes,
		size_t *rowstride);
  struct timeval (*capturetimestamp)(viam_handle_t vh, viam_camera_t camera);
  int (*capturedone)(viam_handle_t vh, viam_camera_t camera);

  int (*queryhwmodes)(viam_handle_t vh, viam_camera_t camera, int offset,
		viam_hwmode_t *supported, size_t n);
  int (*queryhwfeature)(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, unsigned int *methods,
		unsigned int *aumodes);
  int (*sethwfeaturemode)(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, viam_filtermethod_t method,
		viam_filterautomode_t aumode);
  int (*sethwfeaturevalue)(viam_handle_t vh, viam_camera_t camera,
		viam_filtertype_t filter, const viam_filterargs_obj args);
} *viam_acqdev_obj;


/* --- Library generic routines -------------------------------------------- */

int	viam_seterrno(viam_handle_t vh, int eno);

void *	viam_camera_hwdata(viam_camera_t camera);
void	viam_camera_sethwdata(viam_camera_t camera, void *hwdata);


#endif /* H_VIAM_ACQ_PLUGIN */
